﻿using System;
using System.Collections.Generic;
using System.Text;

namespace scgms_mobile.Misc
{
    /// <summary>
    /// Class used as wrapper for received data
    /// </summary>
    public class ChrValueWrapper
    {
        // received bytes to be parsed
        private IList<byte> bytes;
        // current read position
        protected int ReadPos { get; set; }

        /// <summary>
        /// Constructor retaining bytes to be wrapped
        /// </summary>
        /// <param name="src">bytes to be wrapped</param>
        public ChrValueWrapper(byte[] src)
        {
            bytes = new List<byte>(src);
        }

        /// <summary>
        /// Reads single byte from input
        /// </summary>
        /// <returns>Single byte from input</returns>
        public byte ReadByte()
        {
            return bytes[ReadPos++];
        }

        /// <summary>
        /// Reads single unsigned short (16bit) from input
        /// </summary>
        /// <returns>Read unsigned short value</returns>
        public ushort ReadUShort()
        {
            // LSB first
            ushort val = bytes[ReadPos++];
            val |= (ushort)(bytes[ReadPos++] << 8);

            return val;
        }

        /// <summary>
        /// Reads single unsigned int (32bit) from input
        /// </summary>
        /// <returns>Read unsigned int value</returns>
        public uint ReadUInt()
        {
            // LSB first
            uint val = bytes[ReadPos++];
            val |= (uint)(bytes[ReadPos++] << 8);
            val |= (uint)(bytes[ReadPos++] << 16);
            val |= (uint)(bytes[ReadPos++] << 24);

            return val;
        }

        /// <summary>
        /// Reads single signed half-precision IEEE11073 float (16bit) from input
        /// </summary>
        /// <returns>Read IEEE11073 16b float</returns>
        public double ReadSFloat()
        {
            // always start with unsigned short
            ushort val = ReadUShort();

            int exponent = val >> 12;
            int mantissa = val & 0x0FFF;

            // TODO: support Inf, -Inf, NaN

            // exponent is stored in ones complement; high bit = 1 means that the value is negative
            if ((exponent & 0x8) != 0)
                exponent = -(0x10 - exponent);

            // mantissa is stored in ones complement; high bit = 1 means that the value is negative
            if ((mantissa & 0x0800) != 0)
                mantissa = mantissa | 0xF000;

            return mantissa * Math.Pow(10.0, exponent);
        }
    }
}
