﻿using scgms_mobile.Misc;
using Plugin.BLE.Abstractions.Contracts;
using System;
using System.Collections.Generic;
using System.Text;
using System.Threading.Tasks;

namespace scgms_mobile.Devices.Generic
{
    /// <summary>
    ///   Factory class for creating DeviceAdapterBase instances according to recognized device type
    /// </summary>
    public class DeviceAdapterFactory
    {
        /// <summary>
        ///   Builds child of DeviceAdapterBase according to its advertised services
        /// </summary>
        /// <param name="device">device instance to be wrapped</param>
        /// <returns>DeviceAdaptedBase child instance | null</returns>
        public static async Task<DeviceAdapterBase> Build(IDevice device)
        {
            IReadOnlyList<IService> slist = await device.GetServicesAsync();

            foreach (var service in slist)
            {
                if (service.Id.Equals(BLEKnownGuids.IEEEMainServiceGUID))
                    return new IEEE11073.IEEEDevice(device);
                else if (service.Id.Equals(BLEKnownGuids.GuardianMainServiceGUID))
                    return new Guardian.GuardianDevice(device);

                // TODO: more device patterns
            }

            return null;
        }
    }
}
