﻿using scgms_mobile.Devices.Generic.Miscellanous;
using scgms_mobile.Misc;
using System;
using System.Collections.Generic;
using System.Text;

namespace scgms_mobile.Devices.Generic
{
    /// <summary>
    ///   Interface of device state and value changes listener
    /// </summary>
    public interface DeviceValueListener
    {
        /// <summary>
        /// Called upon registering listener
        /// </summary>
        /// <param name="device">wrapped device</param>
        void OnListenerRegistered(DeviceAdapterBase device);

        /// <summary>
        /// Called upon unregistering listener
        /// </summary>
        /// <param name="device">wrapped device</param>
        void OnListenerUnregistered(DeviceAdapterBase device);

        /// <summary>
        /// Called upon receiving new measured CGM value
        /// </summary>
        /// <param name="signal">signal type</param>
        /// <param name="value">glucose value</param>
        /// <param name="timestamp">unix timestamp of measured value</param>
        /// <param name="sequenceNo">sequence number of measured value</param>
        /// <param name="validValue">is this value valid?</param>
        void OnNewMeasurement(DeviceSignalType signal, double value, long timestamp, uint sequenceNo, bool validValue);

        /// <summary>
        /// Called upon battery charge value update
        /// </summary>
        /// <param name="value">battery charge in percents, 0 - 100</param>
        void OnBatteryStateChange(double value);

        /// <summary>
        /// Called upon sensor status change
        /// </summary>
        /// <param name="status">new status</param>
        void OnStatusChange(AppSensorStatus status);

        /// <summary>
        /// Called upon device adapter status change
        /// </summary>
        /// <param name="status">new status</param>
        void OnAdapterStatusChange(DeviceAdapterStatus status);

        /// <summary>
        /// Called upon alert from sensor (threshold reached, ..)
        /// </summary>
        /// <param name="alert">alert type</param>
        void OnAlert(uint alert);

        /// <summary>
        /// Called upon the calibration expire date change
        /// </summary>
        /// <param name="validTo">the calibration is valid to this datetime</param>
        void OnCalibrationExpireDateChanged(DateTime? validTo);

        /// <summary>
        /// Called upon the sensor expore date change
        /// </summary>
        /// <param name="validTo">the sensor is valid to this datetime</param>
        void OnSensorExpireDateChanged(DateTime? validTo);

        /// <summary>
        /// Called when calibration result is returned by sensor
        /// </summary>
        /// <param name="result">calibration result</param>
        void OnCalibrationResult(SensorCalibrationResult result);

        /// <summary>
        /// Indicates whether the backfill is in progress or not
        /// </summary>
        /// <param name="running"></param>
        void SetBackfillProcessState(bool running);
    }
}
