﻿using System;
using System.Collections.Generic;
using System.Text;

namespace scgms_mobile.Devices.Guardian
{
    enum Opcodes : byte
    {
        // in
        AUTH_CHALLENGE = 2,             // (   CRIT) 16B signing key (auth proof)
        SYNC_TIME_RESPONSE = 7,         // (   CRIT) 4B timestamp unk, 1B byte unk

        // out crit
        ENABLE_APP_SECURITY = 0,        // (   CRIT) 1B byte AuthLevel, 1B byte unk, 1B byte KeyExchangeMethod, 16B IV, 16B key
        AUTH_CHALLENGE_RESPONSE = 3,    // (   CRIT) 32B data (signed HMAC256)
        CALIBRATION_REQUEST = 4,        // (   CRIT) 2B short mg/dl glucose, 4B int (time?), 1B bool (always 0)
        SET_SENSOR_TYPE = 5,            // (   CRIT) 1B SensorType enum
        SYNC_TIME = 6,                  // (   CRIT) 4B time, 4B time ??

        // out noncrit
        REQUEST_START_COMM = 0,         // (NONCRIT) 8B unknown
        REQUEST_BACKFILL_RANGE = 2,     // (NONCRIT) 4B time from, 4B time to (unix timestamps?) -- maybe sequence numbers
        REQUEST_SENSOR_EXPIRATION = 3,  // (NONCRIT) 0B
        REQUEST_CAL_EXPIRATION = 4,     // (NONCRIT) 0B
    }
    /*
     * Packets without opcodes:
     *      - SensorStatePacket (sensor state characteristics)
     *      - StartCommunicationResponsePacket (start_comms characteristics)
     *      - TimePacket (unknown, sensor expiration/cal expiration?)
     */

    enum GluCosOpcodes : ushort
    {
        ACK_NACK = 1,                   // 3B payload (1B replytype, 2B unk short) | IN
        AUTH_REQUEST = 5,               // 1B payload (auth level) | OUT
        AUTH_CHALLENGE = 6,             // 32B payload | IN
        AUTH_CHALLENGE_RESPONSE = 7,    // 32B payload | OUT
                                        //UNK_IN_B = ???,
        UNK_OUT_D = 24,
        UNK_OUT_E = 22,
        UNK_OUT_F = 29,
        UNK_IN_G = 23,
        UNK_IN_H = 30
    }

    enum AuthLevel : byte
    {
        UNAUTHORIZED = 0,
        LIMITED_GLUCOSE = 1,
        GLUCOSE_AND_RATE_OF_CHANGE = 7,
        FULL_GLUCOSE = 15,
        GLUCOSE_AND_LIMITED_DIAGNOSTICS = 16,
        GLUCOSE_AND_FULL_DIAGNOSTICS = 23,
        GLUCOSE_DIAGNOSTICS_AND_LIMITED_ENGINEERING = 24
    }

    enum GluCosAuthLevel : byte
    {
        UNAUTHORIZED = 0,
        READ_NON_SENSITIVE_DATA = 30
    }

    enum KeyExchangeMethod : byte
    {
        CLEARTEXT = 0,
        ENCRYPTED = 1
    }

    enum SensorStatus : ushort
    {
        SENSOR_CONFIGURATION_NEEDED = 1,
        TIME_SYNC_NEEDED = 2,
        SENSOR_IN_INITIALIZATION = 4,
        SENSOR_END_OF_LIFE = 8,
        SERVER_BATTERY_LOW = 16,
        SERVER_BATTERY_DEAD = 32,
        SENSOR_GLUCOSE_ABOVE_VALID_RANGE = 64,
        SENSOR_GLUCOSE_BELOW_VALID_RANGE = 128,
        CALIBRATION_REQUIRED = 256,
        CHANGE_SENSOR_ERROR = 512,
        CHANGE_SENSOR = 1024,
        SENSOR_ERROR = 2048,
        CHECK_SENSOR_CONNECTION = 4096,
        SENSOR_ERROR_ALERT = 8192,
        CALIBRATION_PENDING = 16384,
        EXTENSION_BIT = 32768
    }

    enum SignalType
    {
	    NO_SIGNAL = 0,
        GENERIC_GLUCOSE = 1,
        ELECTROCHEMICAL_WORK_1 = 65,
        ELECTROCHEMICAL_WORK_2 = 66,
        ELECTROCHEMICAL_WORK_3 = 67,
        ELECTROCHEMICAL_WORK_4 = 68,
        ELECTROCHEMICAL_WORK_5 = 69,
        OPTICAL_SENSOR_1 = 97,
    }

    enum SensorType
    {
        EXISTING = 0,
        NEW = 1,
        UNSPECIFIED = 2
    }

    enum SensorCapabilities
    {
        MULTIPLE_CALIBRATION_FAILURES_ALLOWED = 1,
        SENSOR_GLUCOSE_AND_CALIBRATION_REQUIRED_ALLOWED = 2,
        CONFIGURABLE_SENSOR_LIFE_ALLOWED = 4,
        INSTANT_SENSOR_GLUCOSE_ALLOWED = 8,
        FIXED_WARM_UP_TIME = 16
    }

    enum CalibrationRequestStatus
    {
        SENT = 0,
        RECEIVED = 1,
        TIMEOUT = 2
    }

    enum CalibrationStatus
    {
        CALIBRATION_ACCEPTED = 0,
        CALIBRATION_REJECTED = 1,
        CHANGE_SENSOR = 2,
        CALIBRATION_NOT_CALCULATED = 3
    }

    enum SensorConnectedState
    {
        DISCONNECTED = 0,
        CONNECTED = 1
    }
}
