﻿using scgms_mobile.Misc;
using System;
using System.Collections.Generic;
using System.IO;
using System.Text;

namespace scgms_mobile.Devices.Guardian.Packets
{
    class EnableAppSecurity
    {
        public static byte[] Build(KeyExchangeMethod exchangeMethod, byte[] key, Support.CounterModeCryptoTransform encryptor)
        {
            MemoryStream appSecBuf = new MemoryStream(1 + 1 + 1 + 1 + 16 + 16);
            using (BinaryWriter writer = new BinaryWriter(appSecBuf))
            {
                var iv = new byte[16];
                Array.Copy(encryptor.GetCounter(), iv, 16);

                writer.Write((byte)Opcodes.ENABLE_APP_SECURITY);
                writer.Write((byte)AuthLevel.GLUCOSE_AND_RATE_OF_CHANGE);   // authorization level requested
                writer.Write((byte)1);                                      // unk0 - always set to 1
                writer.Write((byte)exchangeMethod);                         // key exchange method
                if (exchangeMethod == KeyExchangeMethod.CLEARTEXT)
                {
                    writer.Write(iv);                                       // cipher initialization vector
                    writer.Write(key);                                      // generated crypto key
                }
                else if (exchangeMethod == KeyExchangeMethod.ENCRYPTED)
                {
                    byte[] invalue = new byte[16];
                    Array.Copy(key, invalue, 16);
                    byte[] outvalue = new byte[16];
                    encryptor.TransformBlock(invalue, 0, 16, outvalue, 0);

                    writer.Write(iv);                                       // cipher initialization vector (counter in our case)
                    writer.Write(outvalue);                                 // generated crypto key ("cipher.doFinal(secretKey.getEncoded())")

                    StorageMgr.SetSensorCommIV(encryptor.GetCounter());
                }
            }

            return appSecBuf.ToArray();
        }
    }
}
